class CoreProtectRouter:
    """
    A router to control all database operations on models in the
    auth application.
    """
    def db_for_read(self, model, **hints):
        """
        Attempts to read auth models go to auth_db.
        """
        if model._meta.app_label == 'django_coreprotect':
            return 'django_coreprotect'
        return None

    def db_for_write(self, model, **hints):
        """
        Attempts to write django_coreprotect models go to django_coreprotect.
        """
        if model._meta.app_label == 'django_coreprotect':
            return 'django_coreprotect'
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Allow relations if a model in the django_coreprotect app is involved.
        """
        if obj1._meta.app_label == 'django_coreprotect' or \
           obj2._meta.app_label == 'django_coreprotect':
           return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        """
        Make sure the django_coreprotect app only appears in the 'django_coreprotect'
        database.
        """
        if app_label == 'django_coreprotect':
            return db == 'django_coreprotect'
        return None